import os

ENV_PREFIX = 'SPY_'


def get_env(name, default):
    return os.getenv('{prefix}{variable}'.format(prefix=ENV_PREFIX, variable=name), default)


JSON_DEFAULT_LOG_KEYS = ['threadName', 'process', 'args', 'module', 'levelname', 'pathname', 'lineno', 'funcName']
JSON_LOG_KEYS = get_env('JSON_LOG_KEYS', JSON_DEFAULT_LOG_KEYS)
if not isinstance(JSON_LOG_KEYS, list):
    try:
        JSON_LOG_KEYS = JSON_LOG_KEYS.split(',')
    except:
        JSON_LOG_KEYS = JSON_DEFAULT_LOG_KEYS


SHOW_META = get_env('SHOW_META', True)
if isinstance(SHOW_META, str):
    SHOW_META = 'true' in [SHOW_META.lower()]


LOG_FORMATTER_DEBUG = get_env('LOG_FORMATTER_DEBUG', 'autumn')
LOG_FORMATTER_INFO = get_env('LOG_FORMATTER_INFO', 'monokai')
LOG_FORMATTER_WARNING = get_env('LOG_FORMATTER_WARNING', 'fruity')
LOG_FORMATTER_ERROR = get_env('LOG_FORMATTER_ERROR', 'default')
LOG_FORMATTER_CRITICAL = get_env('LOG_FORMATTER_CRITICAL', 'vs')


LOG_LOGGER = get_env('LOG_LOGGER', 'json-flat')
LOG_LEVEL = get_env('LOG_LEVEL', 'WARNING')


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'json': {'()': 'spylogger.formatters.JSONPrettifiedLogFormatter'},
        'json-flat': {'()': 'spylogger.formatters.JSONLogFormatter'},
        'json-src-key': {'()': 'spylogger.formatters.SrcLocationAsKeyLogFormatter'},
        "ugly": {"format": "%(asctime)s %(levelname)s: %(module)s.%(funcName)s:%(lineno)d  [%(message)s]"}
    },
    'handlers': {
        'json': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'json',
            'stream': 'ext://sys.stdout'
        },
        'json-src-key': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'json-src-key',
            'stream': 'ext://sys.stdout'
        },
        'cw': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'json-flat',
            'stream': 'ext://sys.stdout'
        },
        'ugly': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'ugly',
            'stream': 'ext://sys.stdout'
        },
    },
    'loggers': {
        'json-flat': {'level': LOG_LEVEL, 'handlers': ['cw']},
        'json': {'level': LOG_LEVEL, 'handlers': ['json']},
        'json-src-key': {'level': LOG_LEVEL, 'handlers': ['json-src-key']},
        'ugly': {'level': LOG_LEVEL, 'handlers': ['ugly']},
    }
}



from pygments import highlight
from pygments.formatters.terminal256 import Terminal256Formatter
from pygson.json_lexer import JSONLexer

from spylogger import settings
from spylogger.formatters import JSONPrettifiedLogFormatter, BaseLogFormatter


def get_log_style(level):
    if level >= 41:
        return settings.LOG_FORMATTER_CRITICAL
    elif level >= 31:
        return settings.LOG_FORMATTER_ERROR
    elif level >= 21:
        return settings.LOG_FORMATTER_WARNING
    elif level >= 11:
        return settings.LOG_FORMATTER_INFO
    return settings.LOG_FORMATTER_DEBUG


class PrettyJSONLogFormatter(JSONPrettifiedLogFormatter):

    style = settings.LOG_FORMATTER_DEBUG

    def format(self, record):
        self.style = get_log_style(record.levelno)
        return super(PrettyJSONLogFormatter, self).format(record)

    def format_msg(self, logmsg):
        try:
            return highlight(
                self.dumps(logmsg),
                JSONLexer(),
                Terminal256Formatter(style=self.style)
            ).strip()
        except TypeError:
            return logmsg


class NoMetaPrettyJSONLogFormatter(PrettyJSONLogFormatter):

    def format(self, record):
        self.style = get_log_style(record.levelno)
        record.msg = "{msg}\n{hr}".format(
            msg=self.format_msg(self.build_msg(record, False)),
            hr=self.divider
        )
        return BaseLogFormatter.format(self, record)



from setuptools import setup

setup(
    name='spylogger',
    packages=['spylogger'],
    version='1.2.1',
    include_package_data=True,
    description='Python logging library',
    long_description=open('README.rst').read(),
    url='https://github.com/devstuff-io/spylogger',
    author='meganlkm',
    author_email='devstuff.io@gmail.com',
    install_requires=[],
    extras_require={
        'pretty': [
            'Pygments',
            'pygments-json',
            'pygments-solarized'
        ]
    },
    keywords=['logging', 'logs', 'pretty'],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: System :: Logging',
    ]
)
